import os
import re
import splash_pd
from splash_pd import *

CONF_FILE = os.path.expanduser("~") + "/.gentoo_splash_chooserrc"

class splashThemer:
  """
  This is the problem domain used by the gui.
  """
  GRUB_CONF_LOCATION = "/boot/grub/grub.conf"
  #GRUB_EDITOR="kwrite"
  #GRUB_EDITOR = splash_pd.splashThemer.getEditor()
  SPLASH_DIRECTORY="/etc/splash/"

  def __init__(self):
    self.__ALL_THEMES = self.__getAllThemeList()
    self.ALL_RESOLUTIONS = self.__getAllResolutions()
    #self.Grub_Editor = self.getEditor()
        
  def sortResolutionList(list):
    """
    Static method to sorts a list with resolutions. 
        
    It takes 1 Argument - A List.
        
    Returns the sorted List.
    """
    smallList = []
    middleList = []
    bigList = []
    for element in list:
      if len(element) == 7:
        smallList.append(element)
      elif len(element) == 8:
        middleList.append(element)
      else:
        bigList.append(element)
                
    smallList.sort()
    middleList.sort()
    bigList.sort()
        
    for element in middleList:
      smallList.append(element)
    for element in bigList:
      smallList.append(element)
    return smallList
  sortResolutionList = staticmethod(sortResolutionList)
    
  def __getAllThemeList(self):
    """
    Private method to get all splash themes. Used in __init__.
        
    Only argument is self.
        
    Returns a list of the themes.
    """
    themes = os.listdir(splashThemer.SPLASH_DIRECTORY)
    for element in themes:
      if not os.path.isdir(splashThemer.SPLASH_DIRECTORY + element):
        themes.remove(element)
        if os.path.isdir(splashThemer.SPLASH_DIRECTORY + element):
          if not "images" in os.listdir(self.SPLASH_DIRECTORY + element):
            themes.remove(element)
    if "default" in themes:
      themes.remove("default")
    themes.sort()
    return themes
    
  def getThemeList(self):
    """
    Static method to get private list of all themes (__ALL_THEMES).
        
    Only argument is self.
        
    Returns the list of themes, which was created by __getAllThemeList.
    """
    return self.__ALL_THEMES
  getThemeList = staticmethod(getThemeList)
    
  def mountBoot(self):
    """
    Method to mount boot if it isn't mounted already.
    """
    if os.popen("cat /etc/mtab | grep boot").read().strip() == "":
      os.popen("mount /boot")
    
    
  def getResolutions(self,theme):
    """
    Returns the resolutions for a theme.
        
    Arguments are self and the theme.
    """
    path = splashThemer.SPLASH_DIRECTORY + theme
    fileList = os.listdir(path)
        
    pattern = re.compile(r'[0-9]+x[0-9]+')
    resolutionList = []
    for element in fileList:
      if re.search(pattern, element):
        resolutionList.append(re.search(pattern, element).group())
    resolutionList = splashThemer.sortResolutionList(resolutionList)
    return resolutionList
    
    
  def getImage(self,theme, preview):
    """
    Returns the directory path.
        
    Arguments are self, theme and previewName.
    """
    imagesDirectory = splashThemer.SPLASH_DIRECTORY + theme + "/images/"
    image = imagesDirectory + preview
    return image
    
    
  def getPreviewList(self,theme):
    """
    Returns a list of available preview images for a theme.
        
    Arguments are self and the theme.
    """
    imagesDirectory = splashThemer.SPLASH_DIRECTORY + theme + "/images/"
    files = os.listdir(imagesDirectory)
    for element in files:
        if os.path.islink(imagesDirectory + element):
          files.remove(element)
    files.sort()
    return files
  
    
  def activateTheme(self,theme, resolution):
    """
    Mounts boot, removes the symlink /etc/splash/default (if it exists) and makes
    a new symlink from the choosen theme to default. Then it hits the command:
    splash_geninitramfs -v -g /boot/fbsplash -r RESOLUTION  default.
    
    Arguments are self, theme and the resolution.
    """
    self.mountBoot()
    
    src = splashThemer.SPLASH_DIRECTORY + theme
    dst = splashThemer.SPLASH_DIRECTORY + "default"
    
    if os.path.islink(dst):
      os.remove(dst)
    os.symlink(src, dst)
    command = "splash_geninitramfs -v -g /boot/fbsplash -r " + resolution + " default"
    os.popen(command)
    
    
  def __getAllResolutions(self):
    """
    Returns all resolutions. It iterates through every theme, looks up the available
    resolutions and puts it in a list if it is not already in it.
        
    Only argument is self.
    """
    themes = self.getThemeList(self)
    allResolutions = []
    for theme in themes:
      resolutions = self.getResolutions(theme)
      for resolution in resolutions:
        if not resolution in allResolutions:
          allResolutions.append(resolution)
    allResolutions = splashThemer.sortResolutionList(allResolutions)
    return allResolutions
    
    
  def getThemesWithResolution(self,resolution):
    """
    Returns all themes, which support a choosen resolution.
    """
    allThemes = self.getThemeList(self)
    themesWithResolution = []
    for theme in allThemes:
      path = self.SPLASH_DIRECTORY + theme
      fileList = os.listdir(path)
      pattern = re.compile(resolution)
      resolutionList = []
      for element in fileList:
        if re.search(pattern, element):
          resolutionList.append(re.search(pattern, element).group())
      if len(resolutionList) > 0:
        themesWithResolution.append(theme)
    return themesWithResolution
            
  def getEditor(self):
    """
    Gets the favorit Editor in ~./gentoo_splash_chooserrc
    """
    configFileContent = ""
    try:
      configFileContent = file(splash_pd.CONF_FILE).read()
    except IOError:
      try:
        configFile = open(splash_pd.CONF_FILE, "w")
        configFile.write("#Here you can set your prefered Editor \n" + 'Editor="kwrite"')
      finally:
        configFile.close()
        configFileContent = file(splash_pd.CONF_FILE).read()
    conf_parameter = configFileContent.split("Editor=")[1].split('"')
    return conf_parameter[1]
      
      
      
      
if __name__ == "__main__":
  sp = splashThemer()
  print sp.getEditor()
    
